@@ -1,3 +1,125 @@
+const RightTriangle = '&#9658';   // right-facing triangle symbol (collapsed view)
+const DownTriangle = '&#9660';    // downward-facing triangle symbol (expanded view)
+const Lightning = '&#9889;';      // lightning bolt (for optimization)
+const WhiteLightning = `<span style="opacity:0">${Lightning}</span>`; // invisible but same width as lightning bolt
+
+async function isValidApiKey(apiKey) {
+    const response = await fetch('https://api.openai.com/v1/completions', {
+	method: 'GET',
+	headers: {
+	    'Authorization': `Bearer ${apiKey}`
+	}
+    });
+    const data = await response.json();
+    if (data.error && data.error.code === 'invalid_api_key') {
+	return false;
+    } else {
+	return true;
+    }
+}
+
+function checkApiKey(apiKey) {
+    (async () => {
+	window.localStorage.setItem('api-key', apiKey);
+	// If the API key is empty, clear the status indicator.
+	if (apiKey.length === 0) {
+	    document.getElementById('valid-api-key').innerHTML = '';
+	    return;
+	}
+	const response = await fetch('https://api.openai.com/v1/completions', {
+	    method: 'GET',
+	    headers: {
+		'Authorization': `Bearer ${apiKey}`
+	    }
+	});
+	const data = await response.json();
+	if (data.error && data.error.code === 'invalid_api_key') {
+	    document.getElementById('valid-api-key').innerHTML = '⨯';    
+	} else {
+	    document.getElementById('valid-api-key').innerHTML = '&check;';
+	}
+    })();
+}
+
+async function sendPromptToOpenAI(prompt, len, apiKey) {
+    const endpoint = 'https://api.openai.com/v1/completions';
+    const response = await fetch(endpoint, {
+	method: 'POST',
+	headers: {
+	    'Content-Type': 'application/json',
+	    'Authorization': `Bearer ${apiKey}`
+	},
+	body: JSON.stringify({
+	    'model': 'text-davinci-003',
+	    'prompt': prompt,
+	    "temperature": 0.2,
+	    "max_tokens": len,
+	    "top_p": 1,
+	    "frequency_penalty": 0,
+	    "presence_penalty": 0,
+	    // 'response_format': 'url'
+	})
+    });
+
+    const data = await response.json();
+    // Chop off any blank lines in the header.
+    return data.choices[0].text.replace(/^\s*[\r\n]/gm, '');
+}
+
+function countSpaces(str) {
+  // Use a regular expression to match any whitespace character at the start of the string
+  const match = str.match(/^\s+/);
+
+  // If there was a match, return the length of the match
+  if (match) {
+    return match[0].length;
+  }
+
+  // Otherwise, return 0
+  return 0;
+}
+
+  // Push the final line to the reformatted lines array
+async function optimizeCode(code) {
+    const apiKey = document.getElementById('api-key').value;
+    if (!apiKey) {
+	alert('To activate proposed optimizations, enter an OpenAI API key in advanced options.');
+	return null;
+    }
+    const prompt =  `Below is some Python code to optimize:\n\n${code}\n\nRewrite the above Python code to make it more efficient while keeping the same semantics. Use fast native libraries if that would make it faster than pure Python. Your output should only consist of valid Python code. Output only the resulting Python with brief explanations only included as comments prefaced with #. Include a detailed explanatory comment before the code, starting with the text "# Proposed optimization:". Make the code as clear and simple as possible, while also making it as fast and memory-efficient as possible. Use vectorized operations or the GPU whenever it would substantially increase performance, and try to quantify the speedup in terms of orders of magnitude. If the performance is not likely to increase, leave the code unchanged. Your output should only consist of legal Python code. Format all comments to be less than 40 columns wide:\n\n`;
+    return await sendPromptToOpenAI(prompt, code.length * 4, apiKey);
+}
+
+function proposeOptimization(filename, file_number, lineno) {
+    const prof = globalThis.profile;
+    const this_file = prof.files[filename].lines;
+    const code_line = this_file[lineno-1]['line'];
+    // Count the number of leading spaces to match indentation level on output
+    let leadingSpaceCount = countSpaces(code_line) + 2; // including the lightning bolt
+    let indent = WhiteLightning + '&nbsp;'.repeat(leadingSpaceCount - 1);
+    const elt = document.getElementById(`code-${file_number}-${lineno}`);
+    (async () => {
+	const isValid = await isValidApiKey(document.getElementById("api-key").value);
+	if (!isValid) {
+	    alert('You must enter a valid OpenAI API key to activate proposed optimizations.');
+	    return;
+	}
+	elt.innerHTML = `<em>${indent}working...</em>`;
+	let message = await optimizeCode(code_line);
+	if (!message) {
+	    elt.innerHTML = '';
+	    return;
+	}
+	// Canonicalize newlines
+	message = message.replace(new RegExp('\r?\n','g'), '\n');
+	// Indent every line and format it
+	const formattedCode = message.split('\n')
+	      .map((line) => indent + Prism.highlight(line, Prism.languages.python, "python"))
+	      .join('<br />');
+	elt.innerHTML = `<hr>${formattedCode}`;
+    })();
+}
+
 function memory_consumed_str(size_in_mb) {
   // Return a string corresponding to amount of memory consumed.
   let gigabytes = Math.floor(size_in_mb / 1024);
@@ -445,79 +567,122 @@ function makeTableHeader(fname, gpu, memory, functions = false) {
   return s;
 }
 
+function hideEmptyProfiles() {
+    const elts = document.getElementsByClassName('empty-profile');
+    for (elt of elts) {
+	s = elt.style;
+	s.display = 'none';
+    }
+}
+
+function toggleReduced() {
+    const elts = document.getElementsByClassName('empty-profile');
+    for (elt of elts) {
+	s = elt.style;
+	if (s.display == '') {
+	    s.display = 'none';
+	} else {
+	    s.display = '';
+	}
+    }
+}
+
 function makeProfileLine(
   line,
-  filename,
+    filename,
+    file_number,
   prof,
   cpu_bars,
   memory_bars,
   memory_sparklines,
   memory_activity,
-  gpu_pies
+    gpu_pies,
+    propose_optimizations
 ) {
-  let s = "";
-  s += "<tr>";
-  const total_time =
-    line.n_cpu_percent_python + line.n_cpu_percent_c + line.n_sys_percent;
+    const total_time =
+	  line.n_cpu_percent_python + line.n_cpu_percent_c + line.n_sys_percent;
+    // Disable optimization proposals for low CPU runtime lines.
+    if (total_time < 1.0) {
+	propose_optimizations = false;
+    }
+    const has_memory_results = line.n_avg_mb + line.n_peak_mb + line.memory_samples.length + (line.n_usage_fraction >= 0.01);
+    const has_gpu_results = line.n_gpu_percent >= 1.0;
+    let s = "";
+    if (total_time || has_memory_results || has_gpu_results) {
+	s += "<tr>";
+    } else {
+	s += "<tr class='empty-profile'>";
+    }
   const total_time_str = String(total_time.toFixed(1)).padStart(10, " ");
   s += `<td style="height: 20; width: 100; vertical-align: middle" align="left" data-sort='${total_time_str}'>`;
-  s += `<span style="height: 20; width: 100; vertical-align: middle" id="cpu_bar${cpu_bars.length}"></span>`;
-  cpu_bars.push(
-    makeBar(line.n_cpu_percent_python, line.n_cpu_percent_c, line.n_sys_percent)
-  );
-  if (prof.memory) {
-    s += `<td style="height: 20; width: 100; vertical-align: middle" align="left" data-sort='${String(
+    s += `<span style="height: 20; width: 100; vertical-align: middle" id="cpu_bar${cpu_bars.length}"></span>`;
+    if (total_time) {
+	cpu_bars.push(
+	    makeBar(line.n_cpu_percent_python, line.n_cpu_percent_c, line.n_sys_percent)
+	);
+    } else {
+	cpu_bars.push(null);
+    }
+    if (prof.memory) {
+	s += `<td style="height: 20; width: 100; vertical-align: middle" align="left" data-sort='${String(
       line.n_avg_mb.toFixed(0)
     ).padStart(10, "0")}'>`;
-    s += `<span style="height: 20; width: 100; vertical-align: middle" id="memory_bar${memory_bars.length}"></span>`;
-    s += "</td>";
-    memory_bars.push(
-      makeMemoryBar(
-        line.n_avg_mb.toFixed(0),
-        "average memory",
-        parseFloat(line.n_python_fraction),
-        prof.max_footprint_mb.toFixed(2),
-        "darkgreen"
-      )
-    );
-    s += `<td style="height: 20; width: 100; vertical-align: middle" align="left" data-sort='${String(
+	s += `<span style="height: 20; width: 100; vertical-align: middle" id="memory_bar${memory_bars.length}"></span>`;
+	s += "</td>";
+	if (line.n_avg_mb) {
+	    memory_bars.push(
+		makeMemoryBar(
+		    line.n_avg_mb.toFixed(0),
+		    "average memory",
+		    parseFloat(line.n_python_fraction),
+		    prof.max_footprint_mb.toFixed(2),
+		    "darkgreen"
+		)
+	    );
+	} else {
+	    memory_bars.push(null);
+	}
+	s += `<td style="height: 20; width: 100; vertical-align: middle" align="left" data-sort='${String(
       line.n_peak_mb.toFixed(0)
     ).padStart(10, "0")}'>`;
-    s += `<span style="height: 20; width: 100; vertical-align: middle" id="memory_bar${memory_bars.length}"></span>`;
-    memory_bars.push(
-      makeMemoryBar(
-        line.n_peak_mb.toFixed(0),
-        "peak memory",
-        parseFloat(line.n_python_fraction),
-        prof.max_footprint_mb.toFixed(2),
-        "darkgreen"
-      )
-    );
-    s += "</td>";
-    s += `<td style='vertical-align: middle; width: 100'><span style="height:25; width: 100; vertical-align: middle" id="memory_sparkline${memory_sparklines.length}"></span>`;
-    s += "</td>";
-    if (line.memory_samples.length > 0) {
-      let leak_velocity = 0;
-      if ("leaks" in prof.files[filename]) {
-        if (line.lineno in prof.files[filename].leaks) {
-          leak_velocity = prof.files[filename].leaks[line.lineno].velocity_mb_s;
-        }
-      }
-      memory_sparklines.push(
-        makeSparkline(
-          line.memory_samples,
-          prof.elapsed_time_sec * 1e9,
-          prof.max_footprint_mb,
-          leak_velocity
-        )
-      );
-    } else {
-      memory_sparklines.push(null);
-    }
+	s += `<span style="height: 20; width: 100; vertical-align: middle" id="memory_bar${memory_bars.length}"></span>`;
+	if (line.n_peak_mb) {
+	    memory_bars.push(
+		makeMemoryBar(
+		    line.n_peak_mb.toFixed(0),
+		    "peak memory",
+		    parseFloat(line.n_python_fraction),
+		    prof.max_footprint_mb.toFixed(2),
+		    "darkgreen"
+		)
+	    );
+	} else {
+	    memory_bars.push(null);
+	}
+	s += "</td>";
+	s += `<td style='vertical-align: middle; width: 100'><span style="height:25; width: 100; vertical-align: middle" id="memory_sparkline${memory_sparklines.length}"></span>`;
+	s += "</td>";
+	if (line.memory_samples.length > 0) {
+	    let leak_velocity = 0;
+	    if ("leaks" in prof.files[filename]) {
+		if (line.lineno in prof.files[filename].leaks) {
+		    leak_velocity = prof.files[filename].leaks[line.lineno].velocity_mb_s;
+		}
+	    }
+	    memory_sparklines.push(
+		makeSparkline(
+		    line.memory_samples,
+		    prof.elapsed_time_sec * 1e9,
+		    prof.max_footprint_mb,
+		    leak_velocity
+		)
+	    );
+	} else {
+	    memory_sparklines.push(null);
+	}
     s += '<td style="width: 100; vertical-align: middle" align="center">';
     if (line.n_usage_fraction >= 0.01) {
-      s += `<span style="height: 20; width: 30; vertical-align: middle" id="memory_activity${memory_activity.length}"></span>`;
-      console.log(line.lineno, line.n_usage_fraction, line.n_python_fraction);
+	s += `<span style="height: 20; width: 30; vertical-align: middle" id="memory_activity${memory_activity.length}"></span>`;
       memory_activity.push(
         makeMemoryPie(
           100 *
@@ -561,9 +726,18 @@ function makeProfileLine(
       }
     }
   }
-  s += `<td align="right" style="vertical-align: middle; width: 50" data-sort="${line.lineno}"><font color="gray" style="font-size: 70%; vertical-align: middle" >${line.lineno}&nbsp;</font></td>`;
-  const codeLine = Prism.highlight(line.line, Prism.languages.python, "python");
-  s += `<td style="height:10" align="left" bgcolor="whitesmoke" style="vertical-align: middle" data-sort="${line.lineno}"><pre style="height: 10; display: inline; white-space: pre-wrap; overflow-x: auto; border: 0px; vertical-align: middle"><code class="language-python">${codeLine}</code></pre></td>`;
+    const empty_profile =  (total_time || has_memory_results || has_gpu_results) ? "" : 'empty-profile';
+    s += `<td align="right" class="dummy ${empty_profile}" style="vertical-align: middle; width: 50" data-sort="${line.lineno}"><font color="gray" style="font-size: 70%; vertical-align: middle" >${line.lineno}&nbsp;</font></td>`;
+
+    const optimizationString = propose_optimizations ? `${Lightning}&nbsp;` : `${WhiteLightning}&nbsp;`;
+    const codeLine = Prism.highlight(line.line, Prism.languages.python, "python");
+    s += `<td style="height:10" align="left" bgcolor="whitesmoke" style="vertical-align: middle" data-sort="${line.lineno}">`;
+    if (propose_optimizations) {
+	s += `<span style="vertical-align: middle; cursor: pointer" onclick="proposeOptimization('${filename}', ${file_number}, ${parseInt(line.lineno)}); event.preventDefault()">${optimizationString}</span>`
+    } else {
+	s += optimizationString;
+    }
+    s += `<pre style="height: 10; display: inline; white-space: pre-wrap; overflow-x: auto; border: 0px; vertical-align: middle"><code class="language-python ${empty_profile}">${codeLine}<span id="code-${file_number}-${line.lineno}" bgcolor="white"></span></code></pre></td>`;    
   s += "</tr>";
   return s;
 }
@@ -586,7 +760,53 @@ function buildAllocationMaps(prof, f) {
   return [averageMallocs, peakMallocs];
 }
 
+// Track all profile ids so we can collapse and expand them en masse.
+let allIDs = [];
+
+function collapseAll() {
+    for (const id of allIds) {
+	collapseDisplay(id);
+    }
+}
+
+function expandAll() {
+    for (const id of allIds) {
+	expandDisplay(id);
+    }
+}
+
+function collapseDisplay(id) {
+    const d = document.getElementById(`profile-${id}`);
+    d.style.display = 'none';
+    document.getElementById(`button-${id}`).innerHTML = RightTriangle;
+}
+
+function expandDisplay(id) {
+    const d = document.getElementById(`profile-${id}`);
+    d.style.display = 'block';
+    document.getElementById(`button-${id}`).innerHTML = DownTriangle;
+}
+
+function toggleDisplay(id) {
+    const d = document.getElementById(`profile-${id}`);
+    if (d.style.display == 'block') {
+	d.style.display = 'none';
+	document.getElementById(`button-${id}`).innerHTML = RightTriangle;
+    } else {
+	d.style.display = 'block';
+	document.getElementById(`button-${id}`).innerHTML = DownTriangle;
+    }
+}
+
 async function display(prof) {
+    // Restore the API key from local storage (if any).
+    const old_key = window.localStorage.getItem('api-key');
+    if (old_key) {
+	document.getElementById("api-key").value = old_key;
+	// Update the status.
+	checkApiKey(old_key);
+    }
+    globalThis.profile = prof;
   let memory_sparklines = [];
   let memory_activity = [];
   let cpu_bars = [];
@@ -594,8 +814,8 @@ async function display(prof) {
   let memory_bars = [];
   let tableID = 0;
   let s = "";
-  s += '<div class="row justify-content-center">';
-  s += '<div class="col-auto">';
+  s += '<span class="row justify-content-center">';
+  s += '<span class="col-auto">';
   s += '<table width="50%" class="table text-center table-condensed">';
   s += "<tr>";
   s += `<td><font style="font-size: small"><b>Time:</b> <font color="darkblue">Python</font> | <font color="#6495ED">native</font> | <font color="blue">system</font><br /></font></td>`;
@@ -670,12 +890,14 @@ async function display(prof) {
   }
 
   s += '<tr><td colspan="10">';
-  s += `<p class="text-center"><font style="font-size: 90%; font-style: italic; font-color: darkgray">hover over bars to see breakdowns; click on <font style="font-variant:small-caps; text-decoration:underline">column headers</font> to sort.</font></p>`;
+  s += `<span class="text-center"><font style="font-size: 90%; font-style: italic; font-color: darkgray">hover over bars to see breakdowns; click on <font style="font-variant:small-caps; text-decoration:underline">column headers</font> to sort.</font></span>`;
   s += "</td></tr>";
   s += "</table>";
-  s += "</div>";
-  s += "</div>";
+  s += "</span>";
+  s += "</span>";
 
+    s += '<br class="text-left"><span style="font-size: 80%; color: blue; cursor : pointer;" onClick="expandAll()">&nbsp;show all</span> | <span style="font-size: 80%; color: blue; cursor : pointer;" onClick="collapseAll()">hide all</span>';
+    s += ` | <span style="font-size: 80%; color: blue" onClick="document.getElementById('reduce-checkbox').click()">only display profiled lines&nbsp;</span><input type="checkbox" id="reduce-checkbox" checked onClick="toggleReduced()" /></br>`;
   s += '<div class="container-fluid">';
 
   // Convert files to an array and sort it in descending order by percent of CPU time.
@@ -685,14 +907,21 @@ async function display(prof) {
   });
 
   // Print profile for each file
-  let fileIteration = 0;
-  for (const ff of files) {
-    s += `<p class="text-left"><font style="font-size: 90%"><code>${
+    let fileIteration = 0;
+    allIds = [];
+    for (const ff of files) {
+	const id = `file-${fileIteration}`;
+	allIds.push(id);
+      s += '<p class="text-left">';
+      s += `<span id="button-${id}" title="Click to show or hide profile." style="cursor: pointer; color: blue" onClick="toggleDisplay('${id}')">`;
+	s += `${DownTriangle}`;
+      s += '</span>';
+      s += `<font style="font-size: 90%"><code>${
       ff[0]
     }</code>: % of time = ${ff[1].percent_cpu_time.toFixed(
       1
     )}% (${time_consumed_str(ff[1].percent_cpu_time / 100.0 * prof.elapsed_time_sec * 1e3)}) out of ${time_consumed_str(prof.elapsed_time_sec * 1e3)}.</font></p>`;
-    s += "<div>";
+	s += `<div style="display: block" id="profile-${id}">`;
     s += `<table class="profile table table-hover table-condensed" id="table-${tableID}">`;
     tableID++;
     s += makeTableHeader(ff[0], prof.gpu, prof.memory, false);
@@ -701,7 +930,9 @@ async function display(prof) {
     let prevLineno = -1;
     for (const l in ff[1].lines) {
       const line = ff[1].lines[l];
-      // Add a space whenever we skip a line.
+	if (false) {
+	    // Disabling spacers
+	// Add a space whenever we skip a line.
       if (line.lineno > prevLineno + 1) {
         s += "<tr>";
         for (let i = 0; i < columns.length; i++) {
@@ -714,16 +945,19 @@ async function display(prof) {
         }">&nbsp;</td>`;
         s += "</tr>";
       }
+	}
       prevLineno = line.lineno;
       s += makeProfileLine(
         line,
-        ff[0],
+          ff[0],
+	  fileIteration,
         prof,
         cpu_bars,
         memory_bars,
         memory_sparklines,
         memory_activity,
-        gpu_pies
+          gpu_pies,
+	  true
       );
     }
     s += "</tbody>";
@@ -738,13 +972,15 @@ async function display(prof) {
         const line = prof.files[ff[0]].functions[l];
         s += makeProfileLine(
           line,
-          ff[0],
+            ff[0],
+	    fileIteration,
           prof,
           cpu_bars,
           memory_bars,
           memory_sparklines,
           memory_activity,
-          gpu_pies
+            gpu_pies,
+	    false // no optimizations here
         );
       }
       s += "</table>";
@@ -753,12 +989,12 @@ async function display(prof) {
     fileIteration++;
     // Insert empty lines between files.
     if (fileIteration < files.length) {
-      s += "<p />&nbsp;<hr><p />&nbsp;<p />";
+      s += "<hr>";
     }
   }
   s += "</div>";
   const p = document.getElementById("profile");
-  p.innerHTML = s;
+    p.innerHTML = s;
 
   // Logic for turning on and off the gray line separators.
 
@@ -804,7 +1040,7 @@ async function display(prof) {
   });
   cpu_bars.forEach((p, index) => {
     if (p) {
-      (async () => {
+	(async () => {
         await vegaEmbed(`#cpu_bar${index}`, p, { actions: false });
       })();
     }
@@ -830,13 +1066,13 @@ async function display(prof) {
       })();
     }
   });
-  window.onload = () => {
+    // Hide all empty profiles by default.
+    hideEmptyProfiles();
     if (prof.program) {
-      document.title = "Scalene - " + prof.program;
+	document.title = "Scalene - " + prof.program;
     } else {
-      document.title = "Scalene";
+	document.title = "Scalene";
     }
-  };
 }
 
 function load(profile) {
