@@ -22,14 +22,18 @@ class ScaleneGPU:
             for i in range(self.__ngpus):
                 handle = pynvml.nvmlDeviceGetHandleByIndex(i)
                 self.__handle.append(handle)
-            self.__has_per_pid_accounting = self.set_accounting_mode()
+            self.__has_per_pid_accounting = self._set_accounting_mode()
             total_load = self.gpu_utilization(self.__pid)
             mem_used = self.gpu_memory_usage(self.__pid)
             # If we make it this far, everything is working, so we can profile GPU usage.
             self.__has_gpu = True
 
+    def disable(self) -> None:
+        """Turn off GPU accounting."""
+        self.__has_gpu = False
+
     def __del__(self) -> None:
-        if self.__has_gpu and not self.__has_per_pid_accounting:
+        if self.has_gpu() and not self.__has_per_pid_accounting:
             print(
                 "NOTE: The GPU is currently running in a mode that can reduce Scalene's accuracy when reporting GPU utilization."
             )
@@ -37,7 +41,7 @@ class ScaleneGPU:
                 "Run once as Administrator or root (i.e., prefixed with `sudo`) to enable per-process GPU accounting."
             )
 
-    def set_accounting_mode(self) -> bool:
+    def _set_accounting_mode(self) -> bool:
         """Returns true iff the accounting mode was set already for all GPUs or is now set."""
         ngpus = self.__ngpus
 
@@ -65,8 +69,9 @@ class ScaleneGPU:
 
     def gpu_utilization(self, pid: int) -> float:
         """Return overall GPU utilization by pid if possible.
-
         Otherwise, returns aggregate utilization across all running processes."""
+        if not self.has_gpu():
+            return 0
         ngpus = self.__ngpus
         accounting_on = self.__has_per_pid_accounting
         utilization = 0
@@ -87,6 +92,8 @@ class ScaleneGPU:
 
     def nvml_reinit(self) -> None:
         """Reinitialize the nvidia wrapper."""
+        if not self.has_gpu():
+            return
         self.__handle = []
         with contextlib.suppress(Exception):
             pynvml.nvmlInit()
@@ -99,6 +106,8 @@ class ScaleneGPU:
     def gpu_memory_usage(self, pid: int) -> float:
         """Returns GPU memory used by the process pid, in MB."""
         # Adapted from https://github.com/gpuopenanalytics/pynvml/issues/21#issuecomment-678808658
+        if not self.has_gpu():
+            return 0
         total_used_GPU_memory = 0
         for i in range(self.__ngpus):
             handle = self.__handle[i]
@@ -112,7 +121,7 @@ class ScaleneGPU:
 
     def get_stats(self) -> Tuple[float, float]:
         """Returns a tuple of (utilization %, memory in use)."""
-        if self.__has_gpu:
+        if self.has_gpu():
             total_load = self.gpu_utilization(self.__pid)
             mem_used = self.gpu_memory_usage(self.__pid)
             return (total_load, mem_used)
