@@ -1,5 +1,4 @@
 import copy
-import linecache
 import os
 import random
 import re
@@ -69,44 +68,26 @@ class ScaleneJSON:
         self.gpu = False
 
     def compress_samples(
-        self, uncompressed_samples: List[Any], max_footprint: float
+        self, samples: List[Any], max_footprint: float
     ) -> List[Any]:
-        # Compress the samples so that the granularity is at least
-        # a certain fraction of the maximum footprint.
-        samples = []
-        granularity = max_footprint / self.memory_granularity_fraction
-
-        last_mem = 0
-        for (t, mem) in uncompressed_samples:
-            if abs(mem - last_mem) >= granularity:
-                # We're above the granularity.
-                # Force all memory amounts to be positive.
-                mem = max(0, mem)
-                # Add a tiny bit of random noise to force different values (for the GUI).
-                mem += abs(random.gauss(0.01, 0.01))
-                # Now we append it and set the last amount to be the
-                # current footprint.
-                samples.append([t, mem])
-                last_mem = mem
-
-        if len(samples) > self.max_sparkline_samples:
-            # Try to reduce the number of samples with the
-            # Ramer-Douglas-Peucker algorithm, which attempts to
-            # preserve the shape of the graph. If that fails to bring
-            # the number of samples below our maximum, randomly
-            # downsample (epsilon calculation from
-            # https://stackoverflow.com/questions/57052434/can-i-guess-the-appropriate-epsilon-for-rdp-ramer-douglas-peucker)
-            epsilon = (len(samples) / (3 * self.max_sparkline_samples)) * 2
-            # print("BEFORE len = ", len(samples))
-            new_samples = rdp(samples, epsilon=epsilon)
-            # print("AFTER len = ", len(new_samples))
-            if len(new_samples) > self.max_sparkline_samples:
-                # We still didn't get enough compression; randomly downsample.
-                new_samples = sorted(
-                    random.sample(new_samples, self.max_sparkline_samples)
-                )
-            samples = new_samples
-
+        if len(samples) <= self.max_sparkline_samples:
+            return samples
+        # Try to reduce the number of samples with the
+        # Ramer-Douglas-Peucker algorithm, which attempts to
+        # preserve the shape of the graph. If that fails to bring
+        # the number of samples below our maximum, randomly
+        # downsample (epsilon calculation from
+        # https://stackoverflow.com/questions/57052434/can-i-guess-the-appropriate-epsilon-for-rdp-ramer-douglas-peucker)
+        epsilon = (len(samples) / (3 * self.max_sparkline_samples)) * 2
+        # print("BEFORE len = ", len(samples))
+        new_samples = rdp(samples, epsilon=epsilon)
+        # print("AFTER len = ", len(new_samples))
+        if len(new_samples) > self.max_sparkline_samples:
+            # We still didn't get enough compression; randomly downsample.
+            new_samples = sorted(
+                random.sample(new_samples, self.max_sparkline_samples)
+            )
+        samples = new_samples
         return samples
 
     # Profile output methods
@@ -116,6 +97,7 @@ class ScaleneJSON:
         fname: Filename,
         fname_print: Filename,
         line_no: LineNumber,
+        line: str,
         stats: ScaleneStatistics,
         profile_this_code: Callable[[Filename, LineNumber], bool],
         profile_memory: bool = False,
@@ -209,7 +191,7 @@ class ScaleneJSON:
 
         return {
             "lineno": line_no,
-            "line": linecache.getline(full_fname, line_no),
+            "line": line,
             "n_cpu_percent_c": n_cpu_percent_c,
             "n_cpu_percent_python": n_cpu_percent_python,
             "n_sys_percent": n_sys_percent,
@@ -236,6 +218,7 @@ class ScaleneJSON:
         python_alias_dir: Path,
         program_path: Path,
         profile_memory: bool = True,
+        reduced_profile: bool = False,
     ) -> Dict[str, Any]:
         """Write the profile out."""
         # Get the children's stats, if any.
@@ -323,7 +306,9 @@ class ScaleneJSON:
         # Instead, write info to disk for the main process to collect.
         if pid:
             stats.output_stats(pid, python_alias_dir)
-            return {}
+            # Return a value to indicate that the stats were successfully 
+            # output to the proper directory
+            return {"is_child": True}
 
         if len(report_files) == 0:
             return {}
@@ -382,7 +367,7 @@ class ScaleneJSON:
             # Print out the the profile for the source, line by line.
             full_fname = os.path.normpath(os.path.join(program_path, fname))
             try:
-                with open(full_fname, "r", encoding="utf-8") as source_file:
+                with open(full_fname, "r") as source_file:
                     code_lines = source_file.readlines()
             except (FileNotFoundError, OSError):
                 continue
@@ -397,20 +382,23 @@ class ScaleneJSON:
                     fname=fname,
                     fname_print=fname_print,
                     line_no=LineNumber(lineno),
+                    line=line,
                     stats=stats,
                     profile_this_code=profile_this_code,
                     profile_memory=profile_memory,
                     force_print=False,
                 )
-                # Only output if the payload for the line is non-zero.
                 if profile_line:
-                    profile_line_copy = copy.copy(profile_line)
-                    del profile_line_copy["line"]
-                    del profile_line_copy["lineno"]
-                    if any(profile_line_copy.values()):
-                        output["files"][fname_print]["lines"].append(
-                            profile_line
-                        )
+                    # When reduced-profile set, only output if the payload for the line is non-zero.
+                    if reduced_profile:
+                        profile_line_copy = copy.copy(profile_line)
+                        del profile_line_copy["line"]
+                        del profile_line_copy["lineno"]
+                        if not any(profile_line_copy.values()):
+                            continue
+                    output["files"][fname_print]["lines"].append(
+                        profile_line
+                    )
 
             fn_stats = stats.build_function_stats(fname)
             # Check CPU samples and memory samples.
@@ -436,14 +424,13 @@ class ScaleneJSON:
                         # accumulated; see
                         # ScaleneStatistics.build_function_stats
                         line_no=LineNumber(1),
+                        line=fn_name, # Set the source line to just the function name.
                         stats=fn_stats,
                         profile_this_code=profile_this_code,
                         profile_memory=profile_memory,
                         force_print=True,
                     )
                     if profile_line:
-                        # Change the source code to just the function name.
-                        profile_line["line"] = fn_name
                         # Fix the line number to point to the first line of the function.
                         profile_line["lineno"] = stats.firstline_map[fn_name]
                         output["files"][fname_print]["functions"].append(
