@@ -13,6 +13,7 @@
     usage help: scalene --help
 
 """
+
 import argparse
 import atexit
 import builtins
@@ -29,9 +30,9 @@ import pathlib
 import platform
 import re
 import signal
-import socketserver
 import stat
 import sys
+import sysconfig
 import tempfile
 import threading
 import time
@@ -95,6 +96,18 @@ def require_python(version: Tuple[int, int]) -> None:
 require_python((MINIMUM_PYTHON_VERSION_MAJOR, MINIMUM_PYTHON_VERSION_MINOR))
 
 
+# These are here to simplify print debugging, a la C.
+class LineNo:
+    def __str__(self):
+        return str(inspect.currentframe().f_back.f_lineno)
+
+class FileName:
+    def __str__(self):
+        return str(inspect.currentframe().f_back.f_code.co_filename)
+
+__LINE__ = LineNo()
+__FILE__ = FileName()
+
 # Scalene fully supports Unix-like operating systems; in
 # particular, Linux, Mac OS X, and WSL 2 (Windows Subsystem for Linux 2 = Ubuntu).
 # It also has partial support for Windows.
@@ -900,7 +913,15 @@ class Scalene:
                 Scalene.__python_alias_dir,
                 Scalene.__program_path,
                 profile_memory=Scalene.__args.memory,
+                reduced_profile=Scalene.__args.reduced_profile,
             )
+            # Since the default value returned for "there are no samples"
+            # is `{}`, we use a sentinel value `{"is_child": True}`
+            # when inside a child process to indicate that there are samples, but they weren't
+            # turned into a JSON file because they'll later
+            # be used by the parent process
+            if "is_child" in json_output:
+                return True
             if not Scalene.__output.output_file:
                 Scalene.__output.output_file = "/dev/stdout"
             with open(Scalene.__output.output_file, "w") as f:
@@ -1314,29 +1335,12 @@ class Scalene:
                 ):
                     freed_last_trigger += 1
             timestamp = time.monotonic_ns() - Scalene.__start_time
-            if False:
-                if len(stats.memory_footprint_samples) > 2:
-                    # Compress the footprints by discarding intermediate
-                    # points along increases and decreases. For example:
-                    # if the new point is an increase over the previous
-                    # point, and that point was also an increase,
-                    # eliminate the previous (intermediate) point.
-                    (t1, prior_y) = stats.memory_footprint_samples[-2]
-                    (t2, last_y) = stats.memory_footprint_samples[-1]
-                    y = stats.current_footprint
-                    if prior_y < last_y < y or prior_y > last_y > y:
-                        # Same direction.
-                        # Replace the previous (intermediate) point.
-                        stats.memory_footprint_samples[-1] = [timestamp, y]
-                    else:
-                        stats.memory_footprint_samples.append([timestamp, y])
-            else:
-                stats.memory_footprint_samples.append(
-                    [
-                        timestamp,
-                        stats.current_footprint,
-                    ]
-                )
+            stats.memory_footprint_samples.append(
+                [
+                    timestamp,
+                    stats.current_footprint,
+                ]
+            )
         after = stats.current_footprint
 
         if freed_last_trigger:
@@ -1432,7 +1436,7 @@ class Scalene:
                 )
 
             stats.per_line_footprint_samples[fname][lineno].append(
-                [time.monotonic_ns() - Scalene.__start_time, curr]
+                [time.monotonic_ns() - Scalene.__start_time, max(0, curr)]
             )
             # If we allocated anything, then mark this as the last triggering malloc
             if allocs > 0:
@@ -1547,12 +1551,12 @@ class Scalene:
             # Don't profile the profiler.
             return False
         # Don't profile the Python libraries, unless overridden by --profile-all
-        if (
-            "site-packages" in filename
-            or f"{os.sep}lib{os.sep}python" in filename
-            or f"{os.sep}anaconda3{os.sep}lib" in filename
-        ) and not Scalene.__args.profile_all:
-            return False
+        resolved_filename = str(pathlib.Path(filename).resolve()).lower()
+        for n in sysconfig.get_scheme_names():
+            for p in sysconfig.get_path_names():
+                libdir = str(pathlib.Path(sysconfig.get_path(p, n)).resolve()).lower()
+                if libdir in resolved_filename:
+                    return False
         # Generic handling follows (when no @profile decorator has been used).
         profile_exclude_list = Scalene.__args.profile_exclude.split(",")
         if any(
@@ -1586,6 +1590,9 @@ class Scalene:
             prof not in filename for prof in profile_only_set
         ):
             return False
+        if filename[0] == "<" and filename[-1] == ">":
+            # Special non-file
+            return False
         # Now we've filtered out any non matches to profile-only patterns.
         # If `profile-all` is specified, profile this file.
         if Scalene.__args.profile_all:
@@ -1874,15 +1881,15 @@ class Scalene:
                 "To invoke Scalene programmatically, see the usage noted in https://github.com/plasma-umass/scalene#using-scalene"
             )
             sys.exit(1)
-        Scalene.__orig_signal(
-            Scalene.__signals.start_profiling_signal,
-            Scalene.start_signal_handler,
-        )
-        Scalene.__orig_signal(
-            Scalene.__signals.stop_profiling_signal,
-            Scalene.stop_signal_handler,
-        )
         if sys.platform != "win32":
+            Scalene.__orig_signal(
+                Scalene.__signals.start_profiling_signal,
+                Scalene.start_signal_handler,
+            )
+            Scalene.__orig_signal(
+                Scalene.__signals.stop_profiling_signal,
+                Scalene.stop_signal_handler,
+            )
             Scalene.__orig_siginterrupt(
                 Scalene.__signals.start_profiling_signal, False
             )
@@ -1940,7 +1947,7 @@ class Scalene:
                     progs.extend((sys.argv[0], __file__))
                 if not progs:
                     raise FileNotFoundError
-                with open(progs[0], "rb") as prog_being_profiled:
+                with open(progs[0], "r") as prog_being_profiled:
                     # Read in the code and compile it.
                     code: Any = ""
                     try:
