@@ -410,6 +410,11 @@ class ScaleneStatistics:
                     self.per_line_footprint_samples,
                     x.per_line_footprint_samples,
                 )
+                # Sorting each of the per_line_footprint_sample lists by time, since per_line_footprint_samples
+                # is sent between processes. Samples are in the format [time, footprint]
+                for filename in self.per_line_footprint_samples:
+                    for lineno in self.per_line_footprint_samples[filename]:
+                        self.per_line_footprint_samples[filename][lineno].sort(key=lambda x : x[0])
                 self.increment_per_line_samples(
                     self.memory_malloc_count, x.memory_malloc_count
                 )
@@ -440,6 +445,9 @@ class ScaleneStatistics:
                     x.total_memory_malloc_samples
                 )
                 self.memory_footprint_samples += x.memory_footprint_samples
+                # Sorting footprint samples by time when sample was taken. 
+                # Samples are in the format [time, footprint]
+                self.memory_footprint_samples.sort(key=lambda x : x[0])
                 for k, val in x.function_map.items():
                     if k in self.function_map:
                         self.function_map[k].update(val)
